<?php
	
GFForms::include_feed_addon_framework();

class GFleadsquared extends GFFeedAddOn {
	
	protected $_version = GF_LeadSquared_VERSION;
	protected $_min_gravityforms_version = '1.9.10.16';
	protected $_slug = 'gravityformsleadsquared';
	protected $_path = 'gravityformsleadsquared/leadsquared.php';
	protected $_full_path = __FILE__;
	protected $_url = 'http://www.leadsquared.com';
	protected $_title = 'Gravity Forms LeadSquared Add-On';
	protected $_short_title = 'LeadSquared';
	protected $api = null;
	protected $fields_transient_name = 'gform_leadsquared_fields';
	private static $_instance = null;

	/* Permissions */
	protected $_capabilities_settings_page = 'gravityforms_leadsquared';
	protected $_capabilities_form_settings = 'gravityforms_leadsquared';
	protected $_capabilities_uninstall = 'gravityforms_leadsquared_uninstall';

	/* Members plugin integration */
	protected $_capabilities = array( 'gravityforms_leadsquared', 'gravityforms_leadsquared_uninstall' );
	
	/**
	 * Get instance of this class.
	 * 
	 * @access public
	 * @static
	 * @return GFleadsquared
	 */
	public static function get_instance() {
		
		if ( self::$_instance == null ) {
			self::$_instance = new self;
		}

		return self::$_instance;
		
	}

	/**
	 * Register needed styles.
	 * 
	 * @access public
	 * @return array $styles
	 */
	public function styles() {
		
		$styles = array(
			array(
				'handle'  => 'gform_leadsquared_form_settings_css',
				'src'     => $this->get_base_url() . '/css/form_settings.css',
				'version' => $this->_version,
				'enqueue' => array(
					array( 'admin_page' => array( 'form_settings' ) ),
				)
			)
		);
		
		return array_merge( parent::styles(), $styles );
		
	}
	/**
	 * Setup plugin settings fields.
	 * 
	 * @access public
	 * @return array
	 */
	public function plugin_settings_fields() {
		
		$description  = '<p>';
		$description .= sprintf(
			esc_html__( 'Capture your contact data from Gravity forms into LeadSquared', 'gravityformsleadsquared' ));
		$description .= '</p>';
						
		return array(
			array(
				'title'       => '',
				'description' => $description,
				'fields'      => array(
					array(
						'name'              => 'AccessKey',
						'label'             => esc_html__( 'Access Key', 'gravityformsleadsquared' ),
						'type'              => 'text',
						'class'             => 'medium',
						'dependency'        => array( 'field' => 'authMode', 'values' => array( '', 'email' ) ),
						'feedback_callback' => array( $this, 'plugin_settings_email_feedback' )
					),
					array(
						'name'              => 'SecretKey',
						'label'             => esc_html__( 'Secret Key', 'gravityformsleadsquared' ),
						'type'              => 'text',
						'class'             => 'medium',
						'dependency'        => array( 'field' => 'authMode', 'values' => array( '', 'email' ) ),
						'feedback_callback' => array( $this, 'plugin_settings_email_feedback' )
					),
					array(
						'type'              => 'save',
						'messages'          => array(
							'success' => esc_html__( 'LeadSquared settings have been updated.', 'gravityformsleadsquared' )
						),
					),
				),
			),
		);
		
	}
	


	/**
	 * Get feedback for email address and password fields.
	 * 
	 * @access public
	 * @param string $value
	 * @param array $field
	 * @return bool|null
	 */
	public function plugin_settings_email_feedback( $value, $field ) {
		
		if ( rgblank( $value ) ) {
			return null;
		}
		
		return $this->initialize_api();
		
	}


	/**
	 * Setup fields for feed settings.
	 * 
	 * @access public
	 * @return array
	 */
	public function feed_settings_fields() {
		
		$feed = ( $this->get_posted_settings() ) ? $this->get_posted_settings() : $this->get_current_feed();
		
		/* Build base fields array. */
		$base_fields = array(
			'title'  => '',
			'fields' => array(
				array(
					'name'           => 'feedName',
					'label'          => esc_html__( 'Feed Name', 'gravityformsleadsquared' ),
					'type'           => 'text',
					'required'       => true,
					'default_value'  => $this->get_default_feed_name(),
					'tooltip'        => '<h6>'. esc_html__( 'Name', 'gravityformsleadsquared' ) .'</h6>' . esc_html__( 'Enter a feed name to uniquely identify this setup.', 'gravityformsleadsquared' )
				),
				array(
					'name'           => 'action',
					'label'          => esc_html__( 'Action', 'gravityformshighrise' ),
					'required'       => true,
					'type'           => 'hidden',
					'default_value'  => 'lead'
				)
			)
		);
	
		/* Build lead fields array. */
		$lead_fields = array(
			'title'      => esc_html__( 'Lead Details', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'       => 'leadCustomFields',
					'label'      => 'LeadSquared Map',
					'type'       => 'dynamic_field_map',
					'field_map'  => $this->get_field_map_for_module( 'Leads', 'dynamic' ),
					'tooltip'    => '<h6>'. esc_html__( 'Map Fields', 'gravityformsleadsquared' ) .'</h6>' . esc_html__( 'Map LeadSquared fields to corresponding gravity from fields', 'gravityformsleadsquared' )
				),
				array(
					'name'       => 'OwnerId',
					'label'      => esc_html__( 'Lead Owner', 'gravityformsleadsquared' ),
					'type'       => 'select',
					'choices'    => $this->get_users_for_feed_setting()
				),
				array(
					'name'       => 'Source',
					'label'      => esc_html__( 'Lead Source', 'gravityformsleadsquared' ),
					'type'       => 'text',
					'default_value'  => 'Gravity Form'
				)
			)
		);

		/* Build conditional logic field array. */
		$conditional_fields = array(
			'title'      => esc_html__( 'Feed Conditional Logic', 'gravityformsleadsquared' ),
			'fields'     => array(
				array(
					'name'           => 'feedCondition',
					'type'           => 'feed_condition',
					'label'          => esc_html__( 'Conditional Logic', 'gravityformsleadsquared' ),
					'checkbox_label' => esc_html__( 'Enable', 'gravityformsleadsquared' ),
					'instructions'   => esc_html__( 'Export to LeadSquared if', 'gravityformsleadsquared' ),
					'tooltip'        => '<h6>' . esc_html__( 'Conditional Logic', 'gravityformsleadsquared' ) . '</h6>' . esc_html__( 'When conditional logic is enabled, form submissions will only be exported to LeadSquared when the condition is met. When disabled, all form submissions will be posted.', 'gravityformsleadsquared' )
				),
				
			)
		);
		
		return array( $base_fields, $lead_fields,  $conditional_fields );

	}

	/**
	 * Set feed creation control.
	 * 
	 * @access public
	 * @return bool
	 */
	public function can_create_feed() {
		
		return $this->initialize_api();
		
	}

	/**
	 * Setup columns for feed list table.
	 * 
	 * @access public
	 * @return array
	 */
	public function feed_list_columns() {
		
		return array(
			'feedName' => esc_html__( 'Name', 'gravityformsleadsquared' ),
			'action'   => esc_html__( 'Action', 'gravityformsleadsquared' )
		);
		
	}

	/**
	 * Get value for action feed list column.
	 * 
	 * @access public
	 * @param array $feed
	 * @return string $action
	 */
	public function get_column_value_action() {
		

			
			return esc_html__( 'Create a New Lead', 'gravityformsleadsquared' );
			

		
	}





	/**
	 * Get fields for a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module (default: null)
	 * @return array $fields
	 */
	public function get_module_fields() {
		
		
		$meta_data = $this->api->get_mete_data();
		
		if ( ! empty( $meta_data ) ) {
			$fields = $meta_data;
		}
		

		return  $fields; 
		
	}

	/**
	 * Get field from a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module
	 * @param string $field_name
	 * @return array $field
	 */
	public function get_module_field( $module, $field_name ) {
		
		$module_fields = $this->get_module_fields( $module );
		
		return rgar( $module_fields, $field_name );
		
	}
	

	/**
	 * Get field map fields for a LeadSquared module.
	 * 
	 * @access public
	 * @param string $module
	 * @param string $field_map_type (default: standard)
	 * @return array $field_map
	 */
	public function get_field_map_for_module() {
		
		$fields    = $this->get_module_fields();
		$field_map = array();
		

		
		foreach ( $fields as $field ) {

			/* Add field to field map. */
			if(!isset($field['IsReadOnly'])){
				$field_map[] = array(
					'label'      => $field['DisplayName'],
					'value'		 => $field['SchemaName']
				);
			}
		}		
		return $field_map;
		
	}

	/**
	 * Sort module fields alphabeically by label.
	 * 
	 * @access public
	 * @param array $a
	 * @param array $b
	 * @return int
	 */
	public function sort_module_fields_by_label( $a, $b ) {
		
		return strcmp( $a['label'], $b['label'] );		
		
	}

	/**
	 * Get LeadSquared users for feed field settings.
	 * 
	 * @access public
	 * @return array $users
	 */
	public function get_users_for_feed_setting() {
		
		$users = array(
			array(
				'label' => esc_html__( '-None-', 'gravityformsleadsquared' ),
				'value' => ''
			)
		);
		
		/* If API instance is not initialized, exit. */
		if ( ! $this->initialize_api() ) {
			
			$this->log_error( __METHOD__ . '(): Unable to get users because API is not initialized.' );
			return $users;
			
		}

		$lsq_users = $this->api->get_users();
		
		if ( ! empty( $lsq_users ) ) {

			foreach ( $lsq_users as $user ) {
				$name = $user['FirstName'].' '.$user['LastName'];
				$users[] = array(
					'label' => $name,
					'value' => $user['ID']	
				);
				
			}
			
		}
		
		return $users;
		
	}

	/**
	 * Process the LeadSquared feed.
	 * 
	 * @access public
	 * @param array $feed
	 * @param array $entry
	 * @param array $form
	 * @return void
	 */
	public function process_feed( $feed, $entry, $form ) {
		
		$this->log_debug( __METHOD__ . '(): Processing feed.' );
		
		/* If API instance is not initialized, exit. */
		if ( ! $this->initialize_api() ) {
		
			$this->log_error( __METHOD__ . '(): Failed to set up the API.' );
			return;
			
		}

		/* Create contact or lead */
		if ( rgars( $feed, 'meta/action' ) === 'lead' ) {			
			$lead_id = $this->create_lead( $feed, $entry, $form );		
		}

	}
	

	
	/**
	 * Create a new lead from a feed.
	 * 
	 * @access public
	 * @param array $feed
	 * @param array $entry
	 * @param array $form
	 * @return int $lead_id
	 */
	public function create_lead( $feed, $entry, $form ) {
		$replace = json_encode($entry);
		/* Create lead object. */
		/* Remove OwnerId if not set. */
		if ( !rgblank( rgars( $feed, 'meta/OwnerId' ) ) ) {
					$lead[] = array('Attribute' => 'OwnerId', 'Value' => rgars( $feed, 'meta/OwnerId' ));
			}
		if ( !rgblank( rgars( $feed, 'meta/Source' ) ) ) {
			$lead[] = array('Attribute' => 'Source', 'Value' => rgars( $feed, 'meta/Source' ));	
		}		
		$mapped_fields  = $this->get_dynamic_field_map_fields( $feed, 'leadCustomFields' );

		$count = 0;
		foreach ($form['fields'] as $field) {
			if($field->type == 'survey') {		
				$count2 = 0;
				foreach ((array) $field->inputs as $input) {	
					foreach ($input as $key => $value) {
						$name = $form['fields'][$count]->inputs[$count2]['name'];
						$label = $form['fields'][$count]->inputs[$count2]['label'] ;	
						$replace = str_replace($name, $label,$replace);
					}
				$count2++;
				}
				$countc = 0;
				foreach ((array) $field->choices as $input) {	
					foreach ($input as $key => $value) {
						$value = $form['fields'][$count]->choices[$countc]['value'];
						$text = $form['fields'][$count]->choices[$countc]['text'];
						$replace = str_replace($value, $text,$replace);
					}
				$countc++;
				}				
			}			

			$count++;
		}
		$entry =  json_decode($replace,true);
		foreach ( $mapped_fields as $field_name => $field_id ) {
			
			$field_value = $this->get_field_value( $form, $entry, $field_id );
			
			if ( rgblank( $field_value ) )
				continue;
			
			if($field_name == 'ProspectID'){
				$ProspectID = $field_value;

			} else {
				$atti = array('Attribute' => $field_name, 'Value' => $field_value );
				
			}
			$lead[] = $atti;
			
		}
		/* Filter lead. */
		$lead = gf_apply_filters( 'gform_leadsquared_lead', $form['id'], $lead, $feed, $entry, $form );
	
		$this->log_debug( __METHOD__ . '(): Creating lead: ' . print_r( $lead, true ) );

		try {
			/* Insert lead record. */
			if(isset($ProspectID)){
				$lead_record = $this->api->insert_record($lead,$ProspectID);
			} else {
				$lead_record = $this->api->insert_record($lead);
			}

		
			if($lead_record['Status'] == 'Success'){
				
				if (array_key_exists("AffectedRows",$lead_record['Message'])){
					
				  $lead_id = $ProspectID;
				  
				  }
				else{
					
				  $lead_id = $lead_record['Message']['Id'];
				  
				  }
				/* Get lead ID of new lead record. */
				
				
				/* Save lead ID to entry meta. */
				gform_update_meta( $entry['id'], 'leadsquared_lead_id', $lead_id );
				
				/* Log that lead was created. */
				$this->log_debug( __METHOD__ . '(): Lead #' . $lead_id . ' created.' );
				
				return $lead_id;
			} else {
				
				$json_data = json_encode($lead,true);
				gform_update_meta( $entry['id'], 'leadsquared_json_data', $json_data );
				
				$error_type = $lead_record['ExceptionType'];
				gform_update_meta( $entry['id'], 'leadsquared_error_type', $error_type );
				
				$error_message = $lead_record['ExceptionMessage'];
				gform_update_meta( $entry['id'], 'leadsquared_error_message', $error_message );	
				
				$this->log_error( __METHOD__ . '(): Could not create Lead; ' . $lead_record['ExceptionMessage'] );
				return null;
			}
		
		} catch ( Exception $e ) {
			
			$this->log_error( __METHOD__ . '(): Could not create Lead; ' . $e->getMessage() );
			
			return null;
			
		}
		
	}

	/**
	 * Initialized LeadSquared API if credentials are valid.
	 * 
	 * @access public
	 * @return bool
	 */
	public function initialize_api() {

		if ( ! is_null( $this->api ) ) {
			return true;
		}
		
		/* Include the API library. */
		if ( ! class_exists( 'LSQ_GF' ) ) {
			require_once 'includes/class-leadsquared.php';
		}

		/* Get the plugin settings */
		$settings = $this->get_plugin_settings();
		

		/* If any of the account information fields are empty, return null. */
		if ( rgblank( $settings['AccessKey'] ) || rgblank( $settings['SecretKey'] ) )
			return null;
			
		$this->log_debug( __METHOD__ . "(): Validating API credentials." );
		
		$leadsquared = new LSQ_GF( $settings['AccessKey'],$settings['SecretKey'] );
		
		try {
			
			/* Run API test. */
			if($leadsquared->user_authentication() == true){
				/* Log that test passed. */
				$this->log_debug( __METHOD__ . '(): API credentials are valid.' );
				
				/* Assign LeadSquared object to the class. */
				$this->api = $leadsquared;
				
				return true;
			} else {
				return false;
			}
			
			
		} catch ( Exception $e ) {
			
			/* Log that test failed. */
			$this->log_error( __METHOD__ . '(): API credentials are invalid; '. $e->getMessage() );			

			return false;
			
		}
		
	}

}
